/*==================================================================
  !
  !  mardrone application AR-Drone for MeeGo

  ! Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
  ! All rights reserved.
  !
  !  Author:Kate Alhola  kate.alhola@nokia.com
  !
  ! GNU Lesser General Public License Usage
  ! This file may be used under the terms of the GNU Lesser
  ! General Public License version 2.1 as published by the Free Software
  ! Foundation and appearing in the file LICENSE.LGPL included in the
  ! packaging of this file.  Please review the following information to
  ! ensure the GNU Lesser General Public License version 2.1 requirements
  ! will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
  !
  !
  !
  *===================================================================*/

#include "gaugehorizon.h"
#include <QDebug>
#include <math.h>

GaugeHorizon::GaugeHorizon(QGraphicsItem *parent) :
    Gauge(parent)
{
    m_horizonLineColor=QColor("white");
    m_horizonSkyColor=QColor("darkBlue");
    m_horizonGroundColor=QColor("darkRed");

    int xo=size().width()/2;
    int yo=size().height()/2;

#define DR 3.141592653/180.0


    int arcTable[]={30,45,60,70,80};

    int i = 0;
    int arc0=130;
    int arc1=150;
    fixo+=QLine(xo,yo-arc0,xo,yo-arc1);
    for (i = 0; i < 5; i++) {
        fixo+=QLine(xo+arc0*cos(arcTable[i]*DR),yo-arc0*sin(arcTable[i]*DR),xo+arc1*cos(arcTable[i]*DR),yo-arc1*sin(arcTable[i]*DR));
        fixo+=QLine(xo-arc0*cos(arcTable[i]*DR),yo-arc0*sin(arcTable[i]*DR),xo-arc1*cos(arcTable[i]*DR),yo-arc1*sin(arcTable[i]*DR));
    };
    plane+=QPoint(-40,15);
    plane+=QPoint(0,5);
    plane+=QPoint(40,15);
    plane+=QPoint(0,0);
    plane+=QPoint(-40,15);

    planel+=QPoint(-140,2);
    planel+=QPoint(-75,2);
    planel+=QPoint(-70,0);
    planel+=QPoint(-75,-2);
    planel+=QPoint(-140,-2);
    planel+=QPoint(-140,2);

    planer+=QPoint(140,2);
    planer+=QPoint(75,2);
    planer+=QPoint(70,0);
    planer+=QPoint(75,-2);
    planer+=QPoint(140,-2);
    planer+=QPoint(140,2);

    zeroTriangle+=QPoint(0,-arc1);
    zeroTriangle+=QPoint(-8,-arc1-16);
    zeroTriangle+=QPoint(8,-arc1-16);
    zeroTriangle+=QPoint(0,-arc1);

    angleTriangle+=QPoint(0,-arc1);
    angleTriangle+=QPoint(-8,-arc1+16);
    angleTriangle+=QPoint(8, -arc1+16);
    angleTriangle+=QPoint(0,-arc1);
};



void GaugeHorizon::paint(QPainter *painter,const QStyleOptionGraphicsItem *option,
                       QWidget *widget)
{
    drawHorizon(painter,-1.0*m_value,5*m_value2);
};
//////////////////////////////////////////////////////////////////////////
void GaugeHorizon::drawHorizon(QPainter *painter, float roll, float pitch)
    {
      QRect update_rect;

      int xo, yo;

      painter->save();
      xo=(size().width()/2)/m_scale;
      yo=(size().height()/2)/m_scale;
      painter->setClipRect(QRect(0,0, size().width(),size().height()));
      painter->setTransform(QTransform().scale(m_scale,m_scale));
      QTransform savematrix= painter->transform ();
//      painter->setClipRect(QRect(0,0, size().width()/m_scale,size().height()/m_scale));

      painter->setTransform(QTransform().translate(xo+scenePos().x()/m_scale, yo+scenePos().y()/m_scale).rotate(roll).translate(0,pitch),true);
//      painter->setTransform(QTransform().translate(xo, yo).rotate(roll).translate(0,pitch),true);

//      qDebug("roll=%f pitch=%f xo=%d yo=%d x=%f y=%f",roll,pitch,xo,yo ,xo+scenePos().x(),yo+scenePos().y());

      QBrush skyBrush(m_horizonSkyColor);
      QBrush groundBrush(m_horizonGroundColor);
      painter->fillRect(QRect(-500,0,size().width()*2/m_scale,size().height()*2/m_scale),groundBrush);
      QBrush bbrush(m_backColor);
      QBrush cbrush(m_color);

      painter->setBrush(bbrush);
      painter->fillRect(QRect(-500,0,size().width()*2/m_scale,-1*size().height()*2/m_scale),skyBrush);

      QPen pen(m_color);
      pen.setWidth(2);
      painter->setPen(pen);
      painter->save();
      painter->setTransform(savematrix);
      painter->setTransform(QTransform().translate(xo+scenePos().x()/m_scale, yo+scenePos().y()/m_scale).rotate(roll),true);

      painter->setClipRect(QRect(-100,-105, 200,210));
      painter->setTransform(QTransform().translate(0,pitch),true);

      for(int i=0;i<90;i++) {
          painter->drawLine(40,    50*i,-40,   50*i);
          painter->drawLine(20,    50*i+25,-20,   50*i+25);
          painter->drawText(43,50*i+5,QString("%1").arg(10*i));
          painter->drawText(-63,50*i+5,QString("%1").arg(10*i));
          painter->drawLine(40,    -50*i,-40,  -50*i);
          painter->drawLine(20,    -50*i-25,-20,  -50*i-25);
          painter->drawText(43,-50*i+5,QString("%1").arg(10*i));
          painter->drawText(-63,-50*i+5,QString("%1").arg(10*i));
      }

      painter->restore();
      painter->setTransform(savematrix);
      painter->setTransform(QTransform().translate(xo+scenePos().x()/m_scale, yo+scenePos().y()/m_scale).rotate(roll),true);

      painter->setBrush(cbrush);
      painter->drawPolygon(angleTriangle);


      painter->setTransform(savematrix);
      painter->setTransform(QTransform().translate(xo+scenePos().x()/m_scale, yo+scenePos().y()/m_scale),true);
      painter->setPen(pen);
      painter->drawLines(fixo);
      painter->drawConvexPolygon(plane);
      painter->drawConvexPolygon(planel);
      painter->drawConvexPolygon(planer);
      painter->drawPolygon(zeroTriangle);
      painter->drawArc(-150,-150,300,300,30*16,120*16);
    //  painter->setTransform(savematrix);
      painter->restore();

}

